#ifndef bqZerO_hh
#define bqZerO_hh

#include <cmath>

typedef long double real;

/* The precision of "real" type. */
#define TARKKUUS 35

#ifndef __USE_GNU
#define pow10l(x) powl(10,x)
#endif

template <typename num>
class num0
{
public:
	num n;
	num0() : n(0) { }
	num0(num nn) : n(nn) { }
	inline operator const num& () const { return n; }
	template <typename n2>
	inline num0<num> operator += (const n2 &e) { n += e; return *this; }
};
template <typename num>
class inum0 : public num0<num>
{
public:
	inline operator num () const { return n; }
};
typedef num0<real> real0;
typedef inum0<int> int0;

class merknum
{
	real r;
	unsigned tarkkuus;
public:
	merknum() : r(0), tarkkuus(TARKKUUS) { }
	merknum(real rr, unsigned tark=TARKKUUS) : r(rr), tarkkuus(tark) { }
	inline void combine(const merknum &p) { if(tarkkuus > p.tarkkuus)tarkkuus = p.tarkkuus; }
	inline merknum operator + (const merknum &p) const { return merknum(r + p.r, tarkkuus > p.tarkkuus ? p.tarkkuus : tarkkuus); }
	inline merknum operator - (const merknum &p) const { return merknum(r - p.r, tarkkuus > p.tarkkuus ? p.tarkkuus : tarkkuus); }
	inline merknum operator * (const merknum &p) const { return merknum(r * p.r, tarkkuus > p.tarkkuus ? p.tarkkuus : tarkkuus); }
	inline merknum operator * (const real &p) const { return merknum(r * p, tarkkuus); }
	inline merknum &operator += (const merknum &p) { r += p.r; combine(p); return *this; }
	inline merknum &operator -= (const merknum &p) { r -= p.r; combine(p); return *this; }
	inline merknum &operator *= (const merknum &p) { r *= p.r; combine(p); return *this; }
	inline merknum &operator /= (const merknum &p) { r /= p.r; combine(p); return *this; }
	inline merknum &operator /= (const real &p) { r /= p; return *this; }
	inline merknum &operator *= (const real &p) { r *= p; return *this; }
	inline merknum operator - () const { return merknum(-r, tarkkuus); }
	inline void negate() { r = -r; }
	inline merknum &operator= (const real &rr) { r=rr; maxprecision(); return *this; }
	inline bool operator== (const real &p) const { return r == p; }
	inline bool operator== (const merknum &p) const { return tarkkuus==p.tarkkuus && r==p.r; }
	inline bool operator< (const merknum &p) const { if(r < p.r)return true; if(r > p.r)return false; return tarkkuus < p.tarkkuus; }
	inline operator real () const { return r; }
	
	inline void be(real rr, unsigned tark=TARKKUUS) { r=rr; tarkkuus=tark; }
	inline void maxprecision() { tarkkuus=TARKKUUS; }
	inline void setprecision(unsigned tark) { tarkkuus=tark; }
	inline unsigned precision() const { return tarkkuus; }
	inline const real &value() const { return r; }
	inline bool valid() const { return tarkkuus != 0; }
};

#endif
